@@ -1,9 +1,8 @@
 import logging
+from importlib import metadata
 from os.path import dirname
 
-from pkg_resources import get_distribution
-
-__version__ = get_distribution('sonusai').version
+__version__ = metadata.version('sonusai')
 BASEDIR = dirname(__file__)
 
 # create logger
