@@ -1,12 +1,13 @@
 """sonusai calc_metric_spenh_targetf
 
-usage: calc_metric_spenh_targetf [-hvtpws] [-i MIXID] (-d PLOC) INPUT
+usage: calc_metric_spenh_targetf [-hvtpws] [-i MIXID] (-d PLOC) [-e WER] INPUT
 
 options:
     -h, --help
     -v, --verbose               Be verbose.
     -i MIXID, --mixid MIXID     Mixture ID(s) to generate. [default: *].
     -d PLOC, --ploc PLOC        Location of SonusAI predict data.
+    -e WER, --wer-method WER    Word-Error-Rate method: deepgram, google, or whisper [default: none]
     -t, --truth-est-mode        Calculate extraction using truth and include metrics.
     -p, --plot                  Enable PDF plots file generation per mixture.
     -w, --wav                   Enable WAV file generation per mixture.
@@ -50,6 +51,7 @@ from dataclasses import dataclass
 from typing import Tuple
 from typing import Union
 
+import matplotlib
 import matplotlib.pyplot as plt
 import numpy as np
 import pandas as pd
@@ -62,6 +64,8 @@ from sonusai.mixture import Location
 from sonusai.mixture import MixtureDatabase
 from sonusai.mixture import Predict
 
+matplotlib.use('SVG')
+
 
 # NOTE: global object is required for run-time performance; using 'partial' is much slower.
 @dataclass
@@ -71,181 +75,89 @@ class MPGlobal:
     truth_est_mode: bool = None
     enable_plot: bool = None
     enable_wav: bool = None
+    wer_method: str = None
 
 
 MP_GLOBAL = MPGlobal()
 
 
-def calc_pcm(s_true: np.ndarray, s_est: np.ndarray, with_log: bool = False) -> np.ndarray:
-    """Calculate pcm ((Phase Constrained Magnitude)) err/oss using all source inputs
-    which are frames x nsrc x bins. These should include a noise to
-    be a complete mixture estimate, i.e. noise_est = mixture - sum-over-n(s_est(:,n,:))
-    should be one of the sources in s_true and s_est.
-    Calculates mean-over-srcs(mean-over-tf(| (|Sr(t,f)|+|Si(t,f)|) - (|Shr(t,f)|+|Shi(t,f)|) |))
-    log is optional if with_log=True (for use as a loss function)
-    Returns:
-      pcm scalar
-    ref:  2022-04-sa-sdr-novel-loss-for-separation
-    """
-
-    # multisrc sa-sdr, inputs must be timesteps x nsrc
-    err = s_true - s_est  # samples x nsrc
-
-    # -10*log10(sumk(||sk||^2) / sumk(||sk - shk||^2)
-    num = np.sum(s_true ** 2)  # sum over samples and sources
-    den = np.sum(err ** 2)
-    if num == 0 and den == 0:
-        ratio = np.inf
-    else:
-        ratio = num / (den + np.finfo(np.float32).eps)
-
-    sa_sdr = 10 * np.log10(ratio)
-
-    return sa_sdr
-
-
-def calc_sa_sdr(s_true: np.ndarray,
-                s_est: np.ndarray,
-                with_scale: bool = False,
-                with_negate: bool = False) -> (np.ndarray, np.ndarray):
-    """Calculate sa-sdr (source-aggregated SDR (signal distortion ratio)) using all source
-    inputs which are samples,nsrc time-domain inputs. These should include a noise to
-    be a complete mixture estimate, i.e. noise_est = mixture - sum-over-n(s_est(:,n,:))
-    should be one of the sources in s_true and s_est.
-    Calculates -10*log10(sumn(||sn||^2) / sumn(||sn - shn||^2)
-    Scaling is optional if with_scale=True, scaling is same as in SI-SDR
-    Negation is optional if with_negate=True (for use as a loss function)
-    Returns:
-      sa_sdr scalar
-    ref:  2022-04-sa-sdr-novel-loss-for-separation
-    """
-
-    if with_scale:
-        # calc 1 x nsrc scaling factors
-        ref_energy = np.sum(s_true ** 2, axis=0, keepdims=True)  # 1,nsrc
-        # if ref_energy is zero, just set scaling to 1.0  # 1,nsrc
-        with np.errstate(divide='ignore', invalid='ignore'):
-            opt_scale = np.sum(s_true * s_est, axis=0, keepdims=True) / ref_energy
-            opt_scale[opt_scale == np.inf] = 1.0
-            opt_scale = np.nan_to_num(opt_scale, nan=1.0)
-        scaled_ref = opt_scale * s_true  # 1 x nsrc
-    else:
-        scaled_ref = s_true
-        opt_scale = np.ones((1, s_true.shape[1]), dtype=float)
-
-    # multisrc sa-sdr, inputs must be timesteps x nsrc
-    err = scaled_ref - s_est  # samples x nsrc
-
-    # -10*log10(sumk(||sk||^2) / sumk(||sk - shk||^2)
-    num = np.sum(s_true ** 2)  # sum over samples and sources
-    den = np.sum(err ** 2)
-    if num == 0 and den == 0:
-        ratio = np.inf
-    else:
-        ratio = num / (den + np.finfo(np.float32).eps)
-
-    sa_sdr = 10 * np.log10(ratio)
-
-    if with_negate:
-        sa_sdr = -sa_sdr  # for use as a loss function
-
-    return sa_sdr, opt_scale
-
-
-def mean_square_error(ytrue: np.ndarray,
-                      ypred: np.ndarray,
+def mean_square_error(hypothesis: np.ndarray,
+                      reference: np.ndarray,
                       squared: bool = False) -> (np.ndarray, np.ndarray, np.ndarray):
-    """Calculate mean square error or root mean square error between ytrue, ypred
+    """Calculate root mean square error or mean square error
 
-    Typical inputs expected to be size frames x classes/bins
-    returns:
-      err    mean over both dimensions
-      err_c  mean-over-dim0 i.e. a value per class/bin
-      err_f  mean-over-dim1 i.e. a value per frame
+    :param hypothesis: [frames, bins]
+    :param reference: [frames, bins]
+    :param squared: calculate mean square rather than root mean square
+    :return: mean, mean per bin, mean per frame
     """
-    sqerr = np.square(ytrue - ypred)
-    err_c = np.mean(sqerr, axis=0)  # mean over frames for value per class
-    err_f = np.mean(sqerr, axis=1)  # mean over classes for value per frame
-    err = np.mean(sqerr)  # mean over all
+    sq_err = np.square(reference - hypothesis)
+
+    # mean over frames for value per bin
+    err_b = np.mean(sq_err, axis=0)
+    # mean over bins for value per frame
+    err_f = np.mean(sq_err, axis=1)
+    # mean over all
+    err = np.mean(sq_err)
+
     if not squared:
-        err_c = np.sqrt(err_c)
+        err_b = np.sqrt(err_b)
         err_f = np.sqrt(err_f)
         err = np.sqrt(err)
 
-    return err, err_c, err_f
+    return err, err_b, err_f
 
 
-def mean_abs_percentage_error(ytrue: np.ndarray, ypred: np.ndarray) -> (np.ndarray, np.ndarray, np.ndarray):
-    """Calculate mean abs percentage error between ytrue, ypred
-    If inputs are complex, calculates average:  mape(real)/2 + mape(imag)/2
+def mean_abs_percentage_error(hypothesis: np.ndarray, reference: np.ndarray) -> (np.ndarray, np.ndarray, np.ndarray):
+    """Calculate mean abs percentage error
 
-    Inputs expected to be size frames x classes/bins
-    returns:
-      err    mean over both dimensions
-      err_c  mean-over-dim0 i.e. a value per class/bin
-      err_f  mean-over-dim1 i.e. a value per frame
+    If inputs are complex, calculates average: mape(real)/2 + mape(imag)/2
+
+    :param hypothesis: [frames, bins]
+    :param reference: [frames, bins]
+    :return: mean, mean per bin, mean per frame
     """
-    if np.iscomplexobj(ytrue) is not True and np.iscomplexobj(ypred) is not True:
-        abserr = 100 * np.abs((ytrue - ypred) / (ytrue + np.finfo(np.float32).eps))
+    if not np.iscomplexobj(reference) and not np.iscomplexobj(hypothesis):
+        abs_err = 100 * np.abs((reference - hypothesis) / (reference + np.finfo(np.float32).eps))
     else:
-        ytrue_r = np.real(ytrue)
-        ytrue_i = np.imag(ytrue)
-        ypred_r = np.real(ypred)
-        ypred_i = np.imag(ypred)
-        abserr_r = 100 * np.abs((ytrue_r - ypred_r) / (ytrue_r + np.finfo(np.float32).eps))
-        abserr_i = 100 * np.abs((ytrue_i - ypred_i) / (ytrue_i + np.finfo(np.float32).eps))
-        abserr = (abserr_r / 2) + (abserr_i / 2)
+        reference_r = np.real(reference)
+        reference_i = np.imag(reference)
+        hypothesis_r = np.real(hypothesis)
+        hypothesis_i = np.imag(hypothesis)
+        abs_err_r = 100 * np.abs((reference_r - hypothesis_r) / (reference_r + np.finfo(np.float32).eps))
+        abs_err_i = 100 * np.abs((reference_i - hypothesis_i) / (reference_i + np.finfo(np.float32).eps))
+        abs_err = (abs_err_r / 2) + (abs_err_i / 2)
 
-    err_c = np.mean(abserr, axis=0)  # mean over frames for value per class
-    err_f = np.mean(abserr, axis=1)  # mean over classes for value per frame
-    err = np.mean(abserr)  # mean over all
+    # mean over frames for value per bin
+    err_b = np.around(np.mean(abs_err, axis=0), 3)
+    # mean over bins for value per frame
+    err_f = np.around(np.mean(abs_err, axis=1), 3)
+    # mean over all
+    err = np.around(np.mean(abs_err), 3)
 
-    return err, err_c, err_f
+    return err, err_b, err_f
 
 
-def log_error(ytrue: np.ndarray, ypred: np.ndarray) -> (np.ndarray, np.ndarray, np.ndarray):
-    """Calculate log error between ytrue, ypred which are complex or real float numpy arrays
+def log_error(reference: np.ndarray, hypothesis: np.ndarray) -> (np.ndarray, np.ndarray, np.ndarray):
+    """Calculate log error
 
-    Typical inputs expected to be size frames x classes/bins
-    returns:
-      logerr    mean over both dimensions, scalar value
-      logerr_c  mean-over-dim0 i.e. a value per class/bin
-      logerr_f  mean-over-dim1 i.e. a value per frame
+    :param reference: complex or real [frames, bins]
+    :param hypothesis: complex or real [frames, bins]
+    :return: mean, mean per bin, mean per frame
     """
-    ytsq = np.real(ytrue * np.conjugate(ytrue))
-    ypsq = np.real(ypred * np.conjugate(ypred))
-    lgerr = abs(10 * np.log10((ytsq + np.finfo(np.float32).eps) / (ypsq + np.finfo(np.float32).eps)))
-    # lgerr = abs(10*np.log10(ytsq / (ypsq + np.finfo(np.float32).eps)+np.finfo(np.float32).eps))
-    logerr_c = np.mean(lgerr, axis=0)  # mean over frames for value per class
-    logerr_f = np.mean(lgerr, axis=1)  # mean over classes for value per frame
-    logerr = np.mean(lgerr)
+    reference_sq = np.real(reference * np.conjugate(reference))
+    hypothesis_sq = np.real(hypothesis * np.conjugate(hypothesis))
+    log_err = abs(10 * np.log10((reference_sq + np.finfo(np.float32).eps) / (hypothesis_sq + np.finfo(np.float32).eps)))
+    # log_err = abs(10 * np.log10(reference_sq / (hypothesis_sq + np.finfo(np.float32).eps) + np.finfo(np.float32).eps))
 
-    return logerr, logerr_c, logerr_f
+    # mean over frames for value per bin
+    err_b = np.around(np.mean(log_err, axis=0), 3)
+    # mean over bins for value per frame
+    err_f = np.around(np.mean(log_err, axis=1), 3)
+    # mean over all
+    err = np.around(np.mean(log_err), 3)
 
-
-def calculate_pesq(speech_ref: np.ndarray, speech_est: np.ndarray, error_value=0.0):
-    """Computes the PESQ score of speech estimate audio vs. the clean speech estimate audio
-
-    Upon error, assigns a value of 0, or user specified value in error_value
-    Argument/s:
-      speech_ref - speech reference audio, np.ndarray vector of samples.
-      speech_est - speech estimated audio, np.ndarray vector of samples.
-    Returns:
-      score - float value between -0.5 to 4.5.
-   """
-    import warnings
-
-    from pesq import pesq
-
-    try:
-        with warnings.catch_warnings():
-            warnings.simplefilter('ignore')
-            score = pesq(16000, speech_ref, speech_est, mode='wb')
-    except Exception as e:
-        logger.debug(f'PESQ error {e}')
-        score = error_value
-
-    return score
+    return err, err_b, err_f
 
 
 def plot_mixpred(mixture: AudioT,
@@ -299,7 +211,6 @@ def plot_pdb_predtruth(predict: np.ndarray,
                        metric: Union[np.ndarray, None] = None,
                        tp_title: str = '') -> plt.figure:
     """Plot predict and optionally truth and a metric in power db, e.g. applies 10*log10(predict)"""
-
     num_plots = 2
     if truth_f is not None:
         num_plots += 1
@@ -351,7 +262,6 @@ def plot_epredtruth(predict: np.ndarray,
                     metric: Union[np.ndarray, None] = None,
                     tp_title: str = '') -> plt.figure:
     """Plot predict spectrogram and waveform and optionally truth and a metric)"""
-
     num_plots = 2
     if truth_f is not None:
         num_plots += 1
@@ -402,6 +312,11 @@ def _process_mixture(mixid: int) -> Tuple[pd.DataFrame, pd.DataFrame, pd.DataFra
     import h5py
 
     from sonusai import SonusAIError
+    from sonusai.metrics import calc_pcm
+    from sonusai.metrics import calc_pesq
+    from sonusai.metrics import calc_sa_sdr
+    from sonusai.metrics import calc_wer
+    from sonusai.utils import calc_asr
     from sonusai.utils import float_to_int16
     from sonusai.utils import unstack_complex
     from sonusai.utils import write_wav
@@ -411,6 +326,7 @@ def _process_mixture(mixid: int) -> Tuple[pd.DataFrame, pd.DataFrame, pd.DataFra
     truth_est_mode = MP_GLOBAL.truth_est_mode
     enable_plot = MP_GLOBAL.enable_plot
     enable_wav = MP_GLOBAL.enable_wav
+    wer_method = MP_GLOBAL.wer_method
 
     # 1) Collect true target, noise, mixture data
     target = mixdb.mixture_target(mixid)
@@ -424,8 +340,23 @@ def _process_mixture(mixid: int) -> Tuple[pd.DataFrame, pd.DataFrame, pd.DataFra
 
     feature, truth_f = mixdb.mixture_ft(mixid, mixture=mixture)
 
-    targetfi = mixdb.inverse_transform(target_f)  # need to use inv-tf to match size & tf properties
-    noisefi = mixdb.inverse_transform(noise_f)  # need to use inv-tf to match size & tf properties
+    # need to use inv-tf to match size & tf properties
+    targetfi = mixdb.inverse_transform(target_f)
+    noisefi = mixdb.inverse_transform(noise_f)
+
+    # Test code for tf-based aaware inv transform
+    # mcf = np.concatenate([np.expand_dims(target_f,0), np.expand_dims(noise_f,0)],axis=0)
+    # ttf = tf_istft_olsa_hanns(mcf, N=mixdb.it_config.N,
+    #                           R=mixdb.it_config.R,
+    #                           BinStart=mixdb.fg.bin_start,
+    #                           BinEnd=mixdb.fg.bin_end)
+    # write_wav('tf_target.wav', audio=float_to_int16(ttf[0,:]))
+    # write_wav('tf_noise.wav', audio=float_to_int16(ttf[1, :]))
+    # to = tf_istft_olsa_hanns(target_f, N=mixdb.it_config.N,
+    #                           R=mixdb.it_config.R,
+    #                           BinStart=mixdb.fg.bin_start,
+    #                           BinEnd=mixdb.fg.bin_end)
+    # write_wav('tf_target_only.wav', audio=float_to_int16(to))
 
     # 2)  Read predict data
     output_name = join(predict_location, mixdb.mixtures[mixid].name)
@@ -458,43 +389,76 @@ def _process_mixture(mixid: int) -> Tuple[pd.DataFrame, pd.DataFrame, pd.DataFra
         noise_truth_est_audio = mixdb.inverse_transform(noise_truth_est_complex)
 
     # 4) Metrics
-    # Mean absolute error (real and imag)
-    cmape_tg, cmape_tg_bin, cmape_tg_frame = mean_abs_percentage_error(truth_f_complex, predict_complex)
+    # Mean absolute percentage error (real and imag)
+    # cmape_tg, cmape_tg_bin, cmape_tg_frame = mean_abs_percentage_error(hypothesis=predict_complex,
+    #                                                                    reference=truth_f_complex)
+
     # Target/Speech logerr - PSD estimation accuracy symmetric mean log-spectral distortion
     lerr_tg, lerr_tg_bin, lerr_tg_frame = log_error(truth_f_complex, predict_complex)
     # Noise logerr - PSD estimation accuracy
     lerr_n, lerr_n_bin, lerr_n_frame = log_error(noise_f, noise_est_complex)
+    # PCM loss metric
+    ytrue_f = np.concatenate((truth_f_complex[:, np.newaxis, :], noise_f[:, np.newaxis, :]), axis=1)
+    ypred_f = np.concatenate((predict_complex[:, np.newaxis, :], noise_est_complex[:, np.newaxis, :]), axis=1)
+    pcm, pcm_bin, pcm_frame = calc_pcm(hypothesis=ypred_f, reference=ytrue_f, with_log=True)
+
     # Noise td logerr
     # lerr_nt, lerr_nt_bin, lerr_nt_frame = log_error(noisefi, noise_truth_est_audio)
 
     # SA-SDR (time-domain source-aggragated SDR)
     ytrue = np.concatenate((targetfi[:, np.newaxis], noisefi[:, np.newaxis]), axis=1)
     ypred = np.concatenate((target_est_wav[:, np.newaxis], noise_est_wav[:, np.newaxis]), axis=1)
-    sa_sdr, _ = calc_sa_sdr(ytrue, ypred)  # note: w/o scale is more pessimistic number
+    # note: w/o scale is more pessimistic number
+    sa_sdr, _ = calc_sa_sdr(hypothesis=ypred, reference=ytrue)
 
     # Speech intelligibility measure - PESQ
-    pesq_speech = calculate_pesq(target, target_est_wav)
-    pesq_mixture = calculate_pesq(target, mixture)
-    pesq_impr = pesq_speech - pesq_mixture  # pesq improvement
-    pesq_impr_pc = pesq_impr / (pesq_mixture + np.finfo(np.float32).eps) * 100  # pesq improvement %
+    if int(mixdb.mixtures[mixid].snr) > -99:
+        pesq_speech = calc_pesq(hypothesis=target_est_wav, reference=target)
+        pesq_mixture = calc_pesq(hypothesis=mixture, reference=target)
+        # pesq improvement
+        pesq_impr = pesq_speech - pesq_mixture
+        # pesq improvement %
+        pesq_impr_pc = pesq_impr / (pesq_mixture + np.finfo(np.float32).eps) * 100
+    else:
+        pesq_speech = 0
+        pesq_mixture = 0
+        pesq_impr_pc = 0
+
+    # Calc WER
+    if wer_method == 'none':
+        wer_mx = float('nan')
+        wer_tge = float('nan')
+        wer_pi = float('nan')
+    else:
+        # i.e., wer_method ='google'
+        asr_tt = calc_asr(target, engine=wer_method)  # target truth
+        asr_mx = calc_asr(mixture, engine=wer_method)
+        asr_tge = calc_asr(target_est_wav, engine=wer_method)
+
+        tmp = calc_wer(asr_mx.text, asr_tt.text)  # mixture wer
+        wer_mx = tmp.wer * 100
+        tmp = calc_wer(asr_tge.text, asr_tt.text)  # target estimate wer
+        wer_tge = tmp.wer * 100
+        wer_pi = (wer_mx - wer_tge) / (wer_mx + 0.1) * 100
 
     # 5) Save per mixture metric results
     # Single row in table of scalar metrics per mixture
-    mtable1_col = ['MXSNR', 'MXPESQ', 'PESQ', 'PESQi%', 'SASDR', 'SPCMAPE%', 'SPLERR', 'NLERR', 'SPFILE', 'NFILE']
+    mtable1_col = ['MXSNR', 'MXPESQ', 'MXWER', 'PESQ', 'PESQi%', 'WER', 'WERi%', 'SASDR', 'PCM', 'SPLERR', 'NLERR',
+                   'SPFILE', 'NFILE']
     ti = mixdb.mixtures[mixid].target_file_index[0]
     ni = mixdb.mixtures[mixid].noise_file_index
-    metr1 = [mixdb.mixtures[mixid].snr, pesq_mixture, pesq_speech, pesq_impr_pc, sa_sdr, cmape_tg, lerr_tg,
-             lerr_n, basename(mixdb.targets[ti].name), basename(mixdb.noises[ni].name)]
+    metr1 = [mixdb.mixtures[mixid].snr, pesq_mixture, wer_mx, pesq_speech, pesq_impr_pc, wer_tge, wer_pi,
+             sa_sdr, pcm, lerr_tg, lerr_n, basename(mixdb.targets[ti].name), basename(mixdb.noises[ni].name)]
     mtab1 = pd.DataFrame([metr1], columns=mtable1_col, index=[mixid])
 
     # Stats of per frame estimation metrics
     efs_table2_col = ['Max', 'Min', 'Avg', 'Median']
-    efs_table2_row = ['SPCMAPE%', 'SPLERR', 'NLERR']
-    metr2 = [[np.max(cmape_tg_frame), np.min(cmape_tg_frame), np.mean(cmape_tg_frame), np.median(cmape_tg_frame)],
+    efs_table2_row = ['PCM', 'SPLERR', 'NLERR']
+    metr2 = [[np.max(pcm_frame), np.min(pcm_frame), np.mean(pcm_frame), np.median(pcm_frame)],
              [np.max(lerr_tg_frame), np.min(lerr_tg_frame), np.mean(lerr_tg_frame), np.median(lerr_tg_frame)],
              [np.max(lerr_n_frame), np.min(lerr_n_frame), np.mean(lerr_n_frame), np.median(lerr_n_frame)]]
     mtab2 = pd.DataFrame(metr2, columns=efs_table2_col, index=efs_table2_row)
-    mtab2flat_col = ['MXSNR', 'SPCMAPE Max', 'SPCMAPE Min', 'SPCMAPE Avg', 'SPCMAPE Median',
+    mtab2flat_col = ['MXSNR', 'PCM Max', 'PCM Min', 'PCM Avg', 'PCM Median',
                      'SPLERR Max', 'SPLERR Min', 'SPLERR Avg', 'SPLERR Median',
                      'NLERR Max', 'NLERR Min', 'NLERR Avg', 'NLERR Median']
     tmp = np.insert(np.array(metr2), 0, mixdb.mixtures[mixid].snr).reshape(1, 13)
@@ -521,30 +485,50 @@ def _process_mixture(mixid: int) -> Tuple[pd.DataFrame, pd.DataFrame, pd.DataFra
     all_metrics_table_4 = None
 
     if truth_est_mode:
-        cmape_tgtr, cmape_tgtr_bin, cmape_tgtr_frame = mean_abs_percentage_error(truth_f_complex,
-                                                                                 target_truth_est_complex)
+        # cmape_tgtr, cmape_tgtr_bin, cmape_tgtr_frame = mean_abs_percentage_error(hypothesis=target_truth_est_complex,
+        #                                                                          reference=truth_f_complex)
+
         # metrics of estimates using truth instead of prediction
         lerr_tgtr, lerr_tgtr_bin, lerr_tgtr_frame = log_error(truth_f_complex, target_truth_est_complex)
         lerr_ntr, lerr_ntr_bin, lerr_ntr_frame = log_error(noise_f, noise_truth_est_complex)
+        ypred_tr_f = np.concatenate((target_truth_est_complex[:, np.newaxis, :],
+                                     noise_truth_est_complex[:, np.newaxis, :]), axis=1)
+        pcm_tr, pcm_tr_bin, pcm_tr_frame = calc_pcm(hypothesis=ypred_tr_f, reference=ytrue_f, with_log=True)
 
         # ytrue = np.concatenate((targetfi[:, np.newaxis], noisefi[:, np.newaxis]), axis=1)
         ypred = np.concatenate((target_truth_est_audio[:, np.newaxis], noise_truth_est_audio[:, np.newaxis]), axis=1)
-        sa_sdr_tr, opt_scale_tr = calc_sa_sdr(ytrue, ypred, with_scale=True)  # scale should be ones
+        # scale should be ones
+        sa_sdr_tr, opt_scale_tr = calc_sa_sdr(hypothesis=ypred, reference=ytrue, with_scale=True)
 
-        pesq_speechtr = calculate_pesq(target, target_truth_est_audio)
-        pesq_impr_sptr = pesq_speechtr - pesq_mixture  # pesq improvement
-        pesq_impr_pctr = pesq_impr_sptr / (pesq_mixture + np.finfo(np.float32).eps) * 100  # pesq improvement %
+        if int(mixdb.mixtures[mixid].snr) > -99:
+            pesq_speechtr = calc_pesq(hypothesis=target_truth_est_audio, reference=target)
+            # pesq improvement
+            pesq_impr_sptr = pesq_speechtr - pesq_mixture
+            # pesq improvement %
+            pesq_impr_pctr = pesq_impr_sptr / (pesq_mixture + np.finfo(np.float32).eps) * 100
+        else:
+            pesq_speechtr = 0
+            pesq_impr_pctr = 0
 
-        mtable3_col = ['MXSNR', 'MXPESQ', 'PESQ', 'PESQi%', 'SASDR', 'SPCMAPE%', 'SPLERR', 'NLERR']
-        metr3 = [mixdb.mixtures[mixid].snr, pesq_mixture, pesq_speechtr, pesq_impr_pctr, sa_sdr_tr,
-                 cmape_tgtr, lerr_tgtr, lerr_ntr]
+        if wer_method == 'none':
+            wer_ttge = float('nan')
+            wer_tip = float('nan')
+        else:
+            asr_ttge = calc_asr(target_truth_est_audio, engine=wer_method)
+            tmp = calc_wer(asr_ttge.text, asr_tt.text)  # target estimate wer
+            wer_ttge = tmp.wer * 100
+            wer_tip = (wer_mx - wer_ttge) / (wer_mx + 0.1) * 100
+
+        mtable3_col = ['MXSNR', 'MXPESQ', 'MXWER', 'PESQ', 'PESQi%', 'WER', 'WERi%', 'SASDR', 'PCM', 'SPLERR', 'NLERR']
+        metr3 = [mixdb.mixtures[mixid].snr, pesq_mixture, wer_mx, pesq_speechtr, pesq_impr_pctr, wer_ttge,
+                 wer_tip, sa_sdr_tr, pcm_tr, lerr_tgtr, lerr_ntr]
         mtab3 = pd.DataFrame([metr3], columns=mtable3_col, index=[mixid])
 
         # Stats of per frame estimation metrics
         efs_table4_col = ['Max', 'Min', 'Avg', 'Median']
-        efs_table4_row = ['SPCMAPE%', 'SPLERR', 'NLERR']
-        metr4 = [[np.max(cmape_tgtr_frame), np.min(cmape_tgtr_frame), np.mean(cmape_tgtr_frame),
-                  np.median(cmape_tgtr_frame)],
+        efs_table4_row = ['PCM', 'SPLERR', 'NLERR']
+        metr4 = [[np.max(pcm_tr_frame), np.min(pcm_tr_frame), np.mean(pcm_tr_frame),
+                  np.median(pcm_tr_frame)],
                  [np.max(lerr_tgtr_frame), np.min(lerr_tgtr_frame), np.mean(lerr_tgtr_frame),
                   np.median(lerr_tgtr_frame)],
                  [np.max(lerr_ntr_frame), np.min(lerr_ntr_frame), np.mean(lerr_ntr_frame),
@@ -563,10 +547,11 @@ def _process_mixture(mixid: int) -> Tuple[pd.DataFrame, pd.DataFrame, pd.DataFra
         # Append to all mixture table
         # mtab4flat_col = ['MXSNR', 'SPLERR Max', 'SPLERR Min', 'SPLERR Avg', 'SPLERR Median',
         #                  'NLERR Max', 'NLERR Min', 'NLERR Avg', 'NLERR Median']
-        mtab4flat_col = ['MXSNR', 'SPCMAPE Max', 'SPCMAPE Min', 'SPCMAPE Avg', 'SPCMAPE Median',
+        mtab4flat_col = ['MXSNR', 'PCM Max', 'PCM Min', 'PCM Avg', 'PCM Median',
                          'SPLERR Max', 'SPLERR Min', 'SPLERR Avg', 'SPLERR Median',
                          'NLERR Max', 'NLERR Min', 'NLERR Avg', 'NLERR Median']
-        tmp = np.insert(np.array(metr4), 0, mixdb.mixtures[mixid].snr).reshape(1, 13)  # Insert MXSNR
+        # Insert MXSNR
+        tmp = np.insert(np.array(metr4), 0, mixdb.mixtures[mixid].snr).reshape(1, 13)
         mtab4_flat = pd.DataFrame(tmp, columns=mtab4flat_col, index=[mixid])
 
         all_metrics_table_3 = mtab3
@@ -578,6 +563,7 @@ def _process_mixture(mixid: int) -> Tuple[pd.DataFrame, pd.DataFrame, pd.DataFra
         write_wav(name=base_name + '_target_est.wav', audio=float_to_int16(target_est_wav))
         write_wav(name=base_name + '_noise_est.wav', audio=float_to_int16(noise_est_wav))
         write_wav(name=base_name + '_target.wav', audio=float_to_int16(target))
+        # write_wav(name=base_name + '_tf_target.wav', audio=float_to_int16(ttf))
         write_wav(name=base_name + '_noise.wav', audio=float_to_int16(noise))
         # debug code to test for perfect reconstruction of the extraction method
         # note both 75% olsa-hanns and 50% olsa-hann modes checked to have perfect reconstruction
@@ -610,13 +596,15 @@ def _process_mixture(mixid: int) -> Tuple[pd.DataFrame, pd.DataFrame, pd.DataFra
         with PdfPages(plot_fname) as pdf:
             # page1 we always have a mixture and prediction, target optional if truth provided
             tfunc_name = mixdb.targets[0].truth_settings[0].function  # first target, assumes all have same
-            if tfunc_name == 'mapped_snr_f':  # leave as unmapped snr
+            if tfunc_name == 'mapped_snr_f':
+                # leave as unmapped snr
                 predplot = predict
                 tfunc_name = mixdb.targets[0].truth_settings[0].function
             elif tfunc_name == 'target_f':
                 predplot = 20 * np.log10(abs(predict_complex) + np.finfo(np.float32).eps)
             else:
-                predplot = 10 * np.log10(predict + np.finfo(np.float32).eps)  # use db scale
+                # use dB scale
+                predplot = 10 * np.log10(predict + np.finfo(np.float32).eps)
                 tfunc_name = tfunc_name + ' (db)'
 
             mixspec = 20 * np.log10(abs(mixture_f) + np.finfo(np.float32).eps)
@@ -686,6 +674,7 @@ def main():
     verbose = args['--verbose']
     mixids = args['--mixid']
     predict_location = args['--ploc']
+    wer_method = args['--wer-method'].lower()
     truth_est_mode = args['--truth-est-mode']
     enable_plot = args['--plot']
     enable_wav = args['--wav']
@@ -698,6 +687,7 @@ def main():
 
     from sonusai import create_file_handler
     from sonusai import initial_log_messages
+    from sonusai import logger
     from sonusai import update_console_handler
     from sonusai.mixture import MixtureDatabase
     from sonusai.utils import p_tqdm_map
@@ -719,6 +709,7 @@ def main():
     MP_GLOBAL.truth_est_mode = truth_est_mode
     MP_GLOBAL.enable_plot = enable_plot
     MP_GLOBAL.enable_wav = enable_wav
+    MP_GLOBAL.wer_method = wer_method
 
     progress = tqdm(total=len(mixids), desc='calc_metric_spenh_targetf')
     all_metrics_tables = p_tqdm_map(_process_mixture, mixids, progress=progress)
@@ -726,25 +717,33 @@ def main():
 
     all_metrics_table_1 = pd.concat([item[0] for item in all_metrics_tables])
     all_metrics_table_2 = pd.concat([item[1] for item in all_metrics_tables])
-    all_metrics_table_3 = pd.concat([item[2] for item in all_metrics_tables])
-    all_metrics_table_4 = pd.concat([item[3] for item in all_metrics_tables])
+    if truth_est_mode:
+        all_metrics_table_3 = pd.concat([item[2] for item in all_metrics_tables])
+        all_metrics_table_4 = pd.concat([item[3] for item in all_metrics_tables])
 
     if not enable_summary:
         return
 
     # 9) Done with mixtures, write out summary metrics
-    # Calculate SNR summary for
+    # Calculate SNR summary avg of each non-random snr
     all_mtab1_sorted = all_metrics_table_1.sort_values(by=['MXSNR', 'SPFILE'])
     all_mtab2_sorted = all_metrics_table_2.sort_values(by=['MXSNR'])
-    mtab_snr_summary = all_mtab1_sorted.query('MXSNR==' + str(mixdb.snrs[0])).mean(numeric_only=True).to_frame().T
-    mtab_snr_summary_em = all_mtab2_sorted.query('MXSNR==' + str(mixdb.snrs[0])).mean(numeric_only=True).to_frame().T
-    for snri in range(1, len(mixdb.snrs)):
-        mtab_snr_summary = pd.concat([mtab_snr_summary,
-                                      all_mtab1_sorted.query('MXSNR==' + str(mixdb.snrs[snri])).mean(
-                                          numeric_only=True).to_frame().T])
-        mtab_snr_summary_em = pd.concat([mtab_snr_summary_em,
-                                         all_mtab2_sorted.query('MXSNR==' + str(mixdb.snrs[snri])).mean(
-                                             numeric_only=True).to_frame().T])
+    mtab_snr_summary = None
+    mtab_snr_summary_em = None
+    for snri in range(0, len(mixdb.snrs)):
+        tmp = all_mtab1_sorted.query('MXSNR==' + str(mixdb.snrs[snri])).mean(numeric_only=True).to_frame().T
+        # avoid nan when subset of mixids specified
+        if ~np.isnan(tmp.iloc[0].to_numpy()[0]).any():
+            mtab_snr_summary = pd.concat([mtab_snr_summary, tmp])
+
+        tmp = all_mtab2_sorted.query('MXSNR==' + str(mixdb.snrs[snri])).mean(numeric_only=True).to_frame().T
+        # avoid nan when subset of mixids specified
+        if ~np.isnan(tmp.iloc[0].to_numpy()[0]).any():
+            mtab_snr_summary_em = pd.concat([mtab_snr_summary_em, tmp])
+
+    # Calculate avg metrics over all mixtures except -99
+    all_mtab1_sorted_nom99 = all_mtab1_sorted[all_mtab1_sorted.MXSNR != -99]
+    all_nom99_mean = all_mtab1_sorted_nom99.mean(numeric_only=True)
 
     if truth_est_mode:
         all_mtab3_sorted = all_metrics_table_3.sort_values(by=['MXSNR'])
@@ -764,7 +763,10 @@ def main():
     num_mix = len(mixids)
     if num_mix > 1:
         with open(join(predict_location, 'metric_spenh_targetf_summary.txt'), 'w') as f:
-            print(f'Speech enhancement metrics avg over each SNR:', file=f)
+            print(f'Speech enhancement metrics avg over all {len(all_mtab1_sorted_nom99)} non -99 SNR mixtures:',
+                  file=f)
+            print(all_nom99_mean.to_frame().T.round(2).to_string(), file=f)
+            print(f'\nSpeech enhancement metrics avg over each SNR:', file=f)
             print(mtab_snr_summary.round(2).to_string(), file=f)
             print('', file=f)
             print(f'Extraction statistics stats avg over each SNR:', file=f)
