@@ -129,7 +129,7 @@ def main():
     initial_log_messages('genft')
 
     logger.info(f'Load mixture database from {location}')
-    mixdb = MixtureDatabase(config=location, show_progress=True)
+    mixdb = MixtureDatabase(config=location, show_progress=True, lazy_load=False)
     mixids = mixdb.mixids_to_list(mixids)
 
     total_samples = mixdb.total_samples(mixids)
