@@ -143,6 +143,7 @@ def genmixdb(location: Location,
              show_progress: bool = False,
              test_mode: bool = False) -> MixtureDatabase:
     from random import seed
+    from random import randint
 
     import numpy as np
     import yaml
@@ -167,6 +168,7 @@ def genmixdb(location: Location,
     from sonusai.mixture import get_ir_files
     from sonusai.mixture import get_mixups
     from sonusai.mixture import get_noise_files
+    from sonusai.mixture import get_spectral_masks
     from sonusai.mixture import get_target_files
     from sonusai.mixture import load_config
     from sonusai.mixture import log_duration_and_sizes
@@ -237,6 +239,13 @@ def genmixdb(location: Location,
         logger.debug(f'Random SNRs: {config["random_snrs"]}\n')
         logger.debug(f'Exhaustive noise: {config["exhaustive_noise"]}\n')
 
+    spectral_masks = get_spectral_masks(config)
+    if logging:
+        logger.debug(f'Spectral masks:')
+        for spectral_mask in spectral_masks:
+            logger.debug(spectral_mask)
+        logger.debug('')
+
     if config['truth_mode'] not in ['normal', 'mutex']:
         raise SonusAIError(f'invalid truth_mode: {config["truth_mode"]}')
     truth_mutex = config['truth_mode'] == 'mutex'
@@ -263,12 +272,14 @@ def genmixdb(location: Location,
             random_snrs=config['random_snrs'],
             seed=config['seed'],
             snrs=config['snrs'],
+            spectral_masks=spectral_masks,
             target_augmentations=target_augmentations,
             targets=target_files,
             truth_mutex=truth_mutex,
             truth_reduction_function=config['truth_reduction_function'],
             truth_settings=dataclass_from_dict(TruthSettings, update_truth_settings(config['truth_settings']))),
-        show_progress=show_progress)
+        show_progress=show_progress,
+        lazy_load=False)
     mixdb.location = location
 
     raw_target_audio_samples = sum([len(x) for x in mixdb.raw_target_audios])
@@ -347,33 +358,38 @@ def genmixdb(location: Location,
                             augmented_targets=augmented_targets,
                             raw_target_audios=mixdb.raw_target_audios)
 
-                        for snr in mixdb.snrs:
-                            mixdb.mixtures.append(MRecord(
-                                name=generate_mixture_filename(len(mixdb.mixtures), mixid_padding),
-                                target_file_index=target_file_index,
-                                target_augmentation_index=target_augmentation_index,
-                                noise_file_index=noise_file_index,
-                                noise_offset=noise_offset,
-                                noise_augmentation_index=noise_augmentation_index,
-                                snr=snr,
-                                random_snr=False))
+                        for spectral_mask_index in range(len(mixdb.spectral_masks)):
+                            for snr in mixdb.snrs:
+                                mixdb.mixtures.append(MRecord(
+                                    name=generate_mixture_filename(len(mixdb.mixtures), mixid_padding),
+                                    target_file_index=target_file_index,
+                                    target_augmentation_index=target_augmentation_index,
+                                    noise_file_index=noise_file_index,
+                                    noise_offset=noise_offset,
+                                    noise_augmentation_index=noise_augmentation_index,
+                                    snr=snr,
+                                    spectral_mask_index=spectral_mask_index,
+                                    spectral_mask_seed=randint(0, np.iinfo('i').max),
+                                    random_snr=False))
 
-                            noise_offset = int((noise_offset + target_length) % noise_length)
-                            used_noise_samples += target_length
+                                noise_offset = int((noise_offset + target_length) % noise_length)
+                                used_noise_samples += target_length
 
-                        for random_snr in mixdb.random_snrs:
-                            mixdb.mixtures.append(MRecord(
-                                name=generate_mixture_filename(len(mixdb.mixtures), mixid_padding),
-                                target_file_index=target_file_index,
-                                target_augmentation_index=target_augmentation_index,
-                                noise_file_index=noise_file_index,
-                                noise_offset=noise_offset,
-                                noise_augmentation_index=noise_augmentation_index,
-                                snr=evaluate_random_rule(random_snr),
-                                random_snr=True))
+                            for random_snr in mixdb.random_snrs:
+                                mixdb.mixtures.append(MRecord(
+                                    name=generate_mixture_filename(len(mixdb.mixtures), mixid_padding),
+                                    target_file_index=target_file_index,
+                                    target_augmentation_index=target_augmentation_index,
+                                    noise_file_index=noise_file_index,
+                                    noise_offset=noise_offset,
+                                    noise_augmentation_index=noise_augmentation_index,
+                                    snr=evaluate_random_rule(random_snr),
+                                    spectral_mask_index=spectral_mask_index,
+                                    spectral_mask_seed=randint(0, np.iinfo('i').max),
+                                    random_snr=True))
 
-                            noise_offset = int((noise_offset + target_length) % noise_length)
-                            used_noise_samples += target_length
+                                noise_offset = int((noise_offset + target_length) % noise_length)
+                                used_noise_samples += target_length
 
     else:
         # Get indices and offsets
@@ -394,49 +410,54 @@ def genmixdb(location: Location,
                     augmented_targets=augmented_targets,
                     raw_target_audios=mixdb.raw_target_audios)
 
-                for snr in mixdb.snrs:
-                    (noise_file_index,
-                     noise_augmentation_index,
-                     noise_offset) = _get_next_noise_offset(target_length=target_length,
-                                                            augmented_noise_audios=mixdb.augmented_noise_audios,
-                                                            noise_file_index=noise_file_index,
-                                                            noise_augmentation_index=noise_augmentation_index,
-                                                            noise_offset=noise_offset)
+                for spectral_mask_index in range(len(mixdb.spectral_masks)):
+                    for snr in mixdb.snrs:
+                        (noise_file_index,
+                         noise_augmentation_index,
+                         noise_offset) = _get_next_noise_offset(target_length=target_length,
+                                                                augmented_noise_audios=mixdb.augmented_noise_audios,
+                                                                noise_file_index=noise_file_index,
+                                                                noise_augmentation_index=noise_augmentation_index,
+                                                                noise_offset=noise_offset)
 
-                    mixdb.mixtures.append(MRecord(
-                        name=generate_mixture_filename(len(mixdb.mixtures), mixid_padding),
-                        target_file_index=target_file_index,
-                        target_augmentation_index=target_augmentation_index,
-                        noise_file_index=noise_file_index,
-                        noise_augmentation_index=noise_augmentation_index,
-                        noise_offset=noise_offset,
-                        snr=snr,
-                        random_snr=False))
+                        mixdb.mixtures.append(MRecord(
+                            name=generate_mixture_filename(len(mixdb.mixtures), mixid_padding),
+                            target_file_index=target_file_index,
+                            target_augmentation_index=target_augmentation_index,
+                            noise_file_index=noise_file_index,
+                            noise_augmentation_index=noise_augmentation_index,
+                            noise_offset=noise_offset,
+                            snr=snr,
+                            spectral_mask_index=spectral_mask_index,
+                            spectral_mask_seed=randint(0, np.iinfo('i').max),
+                            random_snr=False))
 
-                    noise_offset += target_length
-                    used_noise_samples += target_length
+                        noise_offset += target_length
+                        used_noise_samples += target_length
 
-                for random_snr in mixdb.random_snrs:
-                    (noise_file_index,
-                     noise_augmentation_index,
-                     noise_offset) = _get_next_noise_offset(target_length=target_length,
-                                                            augmented_noise_audios=mixdb.augmented_noise_audios,
-                                                            noise_file_index=noise_file_index,
-                                                            noise_augmentation_index=noise_augmentation_index,
-                                                            noise_offset=noise_offset)
+                    for random_snr in mixdb.random_snrs:
+                        (noise_file_index,
+                         noise_augmentation_index,
+                         noise_offset) = _get_next_noise_offset(target_length=target_length,
+                                                                augmented_noise_audios=mixdb.augmented_noise_audios,
+                                                                noise_file_index=noise_file_index,
+                                                                noise_augmentation_index=noise_augmentation_index,
+                                                                noise_offset=noise_offset)
 
-                    mixdb.mixtures.append(MRecord(
-                        name=generate_mixture_filename(len(mixdb.mixtures), mixid_padding),
-                        target_file_index=target_file_index,
-                        target_augmentation_index=target_augmentation_index,
-                        noise_file_index=noise_file_index,
-                        noise_augmentation_index=noise_augmentation_index,
-                        noise_offset=noise_offset,
-                        snr=evaluate_random_rule(random_snr),
-                        random_snr=True))
+                        mixdb.mixtures.append(MRecord(
+                            name=generate_mixture_filename(len(mixdb.mixtures), mixid_padding),
+                            target_file_index=target_file_index,
+                            target_augmentation_index=target_augmentation_index,
+                            noise_file_index=noise_file_index,
+                            noise_augmentation_index=noise_augmentation_index,
+                            noise_offset=noise_offset,
+                            snr=evaluate_random_rule(random_snr),
+                            spectral_mask_index=spectral_mask_index,
+                            spectral_mask_seed=randint(0, np.iinfo('i').max),
+                            random_snr=True))
 
-                    noise_offset += target_length
-                    used_noise_samples += target_length
+                        noise_offset += target_length
+                        used_noise_samples += target_length
 
     # Fill in the details
     MP_GLOBAL.mixdb = mixdb
