@@ -43,6 +43,9 @@ Outputs:
     gentcst.log
 
 """
+from typing import Dict
+from typing import List
+
 from sonusai import logger
 
 CONFIG_FILE = 'config.yml'
@@ -52,7 +55,7 @@ def gentcst(fold: str = '*',
             ontology: str = 'ontology.json',
             hierarchical: bool = False,
             update: bool = False,
-            verbose: bool = False) -> (dict, dict):
+            verbose: bool = False) -> (Dict, List[Dict]):
     import os
     from copy import deepcopy
     from os.path import dirname
@@ -135,7 +138,7 @@ def gentcst(fold: str = '*',
                     del val[key]
 
         target = {
-            'name':           file['file'],
+            'name': file['file'],
             'truth_settings': truth_settings
         }
 
@@ -347,18 +350,18 @@ def get_req_folds(folds: list, fold: str) -> list:
 
 
 def get_folds_from_files(files: list) -> list:
-    result = [sub['fold'] for sub in files]
+    result = [file['fold'] for file in files]
     # Converting to set and back to list ensures uniqueness
     return sorted(list(set(result)))
 
 
 def get_leaves_from_files(files: list) -> list:
-    result = [sub['leaf'] for sub in files]
+    result = [file['leaf'] for file in files]
     # Converting to set and back to list ensures uniqueness
     return sorted(list(set(result)))
 
 
-def get_labels_from_files(files: list) -> list:
+def get_labels_from_files(files: List[Dict]) -> List[Dict]:
     all_labels = [file['labels'] for file in files]
 
     # Get labels by depth
@@ -397,7 +400,7 @@ def get_index_from_label(labels: list, label: str) -> int:
     return next((item for item in labels if item['display_name'] == label), None)['index']
 
 
-def get_config() -> dict:
+def get_config() -> Dict:
     from os.path import exists
 
     from sonusai import SonusAIError
@@ -531,9 +534,7 @@ def main():
     if not output_name:
         output_name = basename(getcwd()) + '.yml'
 
-    log_name = 'gentcst.log'
-
-    create_file_handler(log_name)
+    create_file_handler('gentcst.log')
 
     config, labels = gentcst(fold=fold,
                              ontology=ontology,
