@@ -1,6 +1,6 @@
 """sonusai keras_train
 
-usage: keras_train [-hv] (-m MODEL) (-l VLOC) [-w KMODEL] [-e EPOCHS] [-b BATCH] [-t TSTEPS] [-p ESP] TLOC
+usage: keras_train [-hgv] (-m MODEL) (-l VLOC) [-w KMODEL] [-e EPOCHS] [-b BATCH] [-t TSTEPS] [-p ESP] TLOC
 
 options:
     -h, --help
@@ -12,6 +12,7 @@ options:
     -b BATCH, --batch BATCH         Batch size.
     -t TSTEPS, --tsteps TSTEPS      Timesteps.
     -p ESP, --patience ESP          Early stopping patience.
+    -g, --loss-batch-log            Enable per-batch loss log. [default: False]
 
 Use Keras to train a model defined by a Python definition file and SonusAI genft data.
 
@@ -20,11 +21,65 @@ Inputs:
     VLOC    A SonusAI mixture database directory to use for validation data.
 
 Results are written into subdirectory <MODEL>-<TIMESTAMP>.
+Per-batch loss history, if enabled, is written to <basename>-history-lossb.npy
 
 """
+import tensorflow as tf
+
 from sonusai import logger
 
 
+class LossBatchHistory(tf.keras.callbacks.Callback):
+    def __init__(self):
+        super().__init__()
+        self.history = None
+
+    def on_train_begin(self, logs=None):
+        self.history = {'loss': []}
+
+    def on_batch_end(self, batch, logs=None):
+        if logs is None:
+            logs = {}
+        self.history['loss'].append(logs.get('loss'))
+
+
+class SonusAIModelCheckpoint(tf.keras.callbacks.ModelCheckpoint):
+    def __init__(self,
+                 filepath,
+                 monitor: str = "val_loss",
+                 verbose: int = 0,
+                 save_best_only: bool = False,
+                 save_weights_only: bool = False,
+                 mode: str = "auto",
+                 save_freq="epoch",
+                 options=None,
+                 initial_value_threshold=None,
+                 **kwargs):
+        super().__init__(filepath,
+                         monitor,
+                         verbose,
+                         save_best_only,
+                         save_weights_only,
+                         mode,
+                         save_freq,
+                         options,
+                         initial_value_threshold,
+                         **kwargs)
+        self.feature = kwargs.get('feature', None)
+        self.num_classes = kwargs.get('num_classes', None)
+
+    def _save_model(self, epoch, batch, logs):
+        import h5py
+
+        super()._save_model(epoch, batch, logs)
+
+        with h5py.File(self.filepath, 'a') as f:
+            if self.feature is not None:
+                f.attrs['sonusai_feature'] = self.feature
+            if self.num_classes is not None:
+                f.attrs['sonusai_num_classes'] = str(self.num_classes)
+
+
 def main():
     from docopt import docopt
 
@@ -41,6 +96,7 @@ def main():
     batch_size = args['--batch']
     timesteps = args['--tsteps']
     esp = args['--patience']
+    loss_batch_log = args['--loss-batch-log']
     t_name = args['TLOC']
 
     import warnings
@@ -53,7 +109,6 @@ def main():
     import h5py
     import keras_tuner as kt
     import numpy as np
-    import tensorflow as tf
 
     with warnings.catch_warnings():
         warnings.simplefilter('ignore')
@@ -70,6 +125,7 @@ def main():
     from sonusai.utils import import_keras_model
     from sonusai.utils import stratified_shuffle_split_mixid
     from sonusai.utils import reshape_outputs
+    from sonusai.utils import get_frames_per_batch
 
     model_base = basename(model_name)
     model_root = splitext(model_base)[0]
@@ -105,15 +161,18 @@ def main():
 
     # Check overrides
     timesteps = check_keras_overrides(model, t_mixdb.feature, t_mixdb.num_classes, timesteps, batch_size)
+    # Calculate batches per epoch, use ceiling as last batch is zero extended
+    frames_per_batch = get_frames_per_batch(batch_size, timesteps)
+    batches_per_epoch = int(np.ceil(t_mixdb.total_feature_frames('*') / frames_per_batch))
 
-    logger.info('Building model')
+    logger.info('Building and compiling model')
     try:
         hypermodel = model.MyHyperModel(feature=t_mixdb.feature,
                                         num_classes=t_mixdb.num_classes,
                                         timesteps=timesteps,
                                         batch_size=batch_size)
         built_model = hypermodel.build_model(kt.HyperParameters())
-        built_model = hypermodel.compile_default(built_model)
+        built_model = hypermodel.compile_default(built_model, batches_per_epoch)
     except Exception as e:
         logger.exception(f'Error: build_model() in {model_base} failed: {e}')
         raise SystemExit(1)
@@ -179,11 +238,22 @@ def main():
                            verbose=1,
                            patience=int(esp))
 
-    ckpt_callback = tf.keras.callbacks.ModelCheckpoint(filepath=base_name + '-ckpt-weights.h5',
-                                                       save_weights_only=True,
-                                                       monitor='val_loss',
-                                                       mode='min',
-                                                       save_best_only=True)
+    ckpt_callback = SonusAIModelCheckpoint(filepath=base_name + '-ckpt-weights.h5',
+                                           save_weights_only=True,
+                                           monitor='val_loss',
+                                           mode='min',
+                                           save_best_only=True,
+                                           feature=hypermodel.feature,
+                                           num_classes=hypermodel.num_classes)
+
+    csv_logger = tf.keras.callbacks.CSVLogger(base_name + '-history.csv')
+    callbacks = [es, ckpt_callback, csv_logger]
+    # loss_batch_log = True
+    loss_batchlogger = None
+    if loss_batch_log is True:
+        loss_batchlogger = LossBatchHistory()
+        callbacks.append(loss_batchlogger)
+        logger.info(f'Adding per batch loss logging to training')
 
     if weights_name is not None:
         logger.info(f'Loading weights from {weights_name}')
@@ -200,13 +270,17 @@ def main():
                               epochs=epochs,
                               validation_data=v_datagen,
                               shuffle=False,
-                              callbacks=[es, ckpt_callback])
+                              callbacks=callbacks)
 
     # Save history into numpy file
     history_name = base_name + '-history'
     np.save(history_name, history.history)
     # Note: Reload with history=np.load(history_name, allow_pickle='TRUE').item()
     logger.info(f'Saved training history to numpy file {history_name}.npy')
+    if loss_batch_log is True:
+        his_batch_loss_name = base_name + '-history-lossb.npy'
+        np.save(his_batch_loss_name, loss_batchlogger.history)
+        logger.info(f'Saved per-batch loss history to numpy file {his_batch_loss_name}')
 
     # Find checkpoint file and load weights for prediction and model save
     checkpoint_name = None
