@@ -16,6 +16,7 @@ The sonusai commands are:
    mkwav                        Make WAV files from a mixture database
    onnx_predict                 Run ONNX predict on a trained model
    plot                         Plot mixture data
+   post_spenh_targetf           Run post-processing for speech enhancement targetf data
    tplot                        Plot truth data
    vars                         List custom SonusAI variables
 
@@ -50,6 +51,7 @@ def main():
         'mkwav',
         'onnx_predict',
         'plot',
+        'post_spenh_targetf',
         'tplot',
         'vars',
     )
