@@ -22,6 +22,7 @@ from sonusai.mixture.config import get_default_config
 from sonusai.mixture.config import get_ir_files
 from sonusai.mixture.config import get_max_class
 from sonusai.mixture.config import get_noise_files
+from sonusai.mixture.config import get_spectral_masks
 from sonusai.mixture.config import get_target_files
 from sonusai.mixture.config import load_config
 from sonusai.mixture.config import raw_load_config
@@ -54,6 +55,7 @@ from sonusai.mixture.mixdb import MixtureDatabase
 from sonusai.mixture.mixdb import MixtureDatabaseConfig
 from sonusai.mixture.mixdb import generic_ids_to_list
 from sonusai.mixture.mixdb import get_mrecords_from_mixids
+from sonusai.mixture.spectral_mask import apply_spectral_mask
 from sonusai.mixture.target_class_balancing import balance_targets
 from sonusai.mixture.targets import get_augmented_target_indices_by_class
 from sonusai.mixture.targets import get_augmented_target_indices_for_mixup
@@ -81,6 +83,8 @@ from sonusai.mixture.types import ListAudiosT
 from sonusai.mixture.types import Location
 from sonusai.mixture.types import Predict
 from sonusai.mixture.types import Segsnr
+from sonusai.mixture.types import SpectralMask
+from sonusai.mixture.types import SpectralMasks
 from sonusai.mixture.types import TargetFile
 from sonusai.mixture.types import TargetFiles
 from sonusai.mixture.types import Truth
