@@ -6,6 +6,7 @@ from typing import Union
 from sonusai.mixture.types import ImpulseResponseFiles
 from sonusai.mixture.types import Location
 from sonusai.mixture.types import NoiseFiles
+from sonusai.mixture.types import SpectralMasks
 from sonusai.mixture.types import TargetFiles
 
 
@@ -49,7 +50,7 @@ def load_config(name: Location) -> Dict:
 
 
 def update_config_from_file(name: Location, config: Dict) -> Dict:
-    """Update the given config with the config in the given YAML file
+    """Update the given config with the config in the YAML file
 
     :param name: File name
     :param config: Config dictionary to update
@@ -61,44 +62,48 @@ def update_config_from_file(name: Location, config: Dict) -> Dict:
     from sonusai.mixture import REQUIRED_CONFIGS
     from sonusai.mixture import VALID_CONFIGS
 
-    new_config = deepcopy(config)
+    updated_config = deepcopy(config)
 
     try:
-        given_config = raw_load_config(name)
+        new_config = raw_load_config(name)
     except Exception as e:
         raise SonusAIError(f'Error loading config from {name}: {e}')
 
     # Check for unrecognized keys
-    for key in given_config:
+    for key in new_config:
         if key not in VALID_CONFIGS:
             nice_list = '\n'.join([f'  {item}' for item in VALID_CONFIGS])
             raise SonusAIError(f'Invalid config parameter in {name}: {key}.\nValid config parameters are:\n{nice_list}')
 
     # Use default config as base and overwrite with given config keys as found
-    for key in new_config:
-        if key in given_config:
+    for key in updated_config:
+        if key in new_config:
             if key not in ['truth_settings']:
-                new_config[key] = given_config[key]
+                updated_config[key] = new_config[key]
 
     # Handle 'truth_settings' special case
-    if 'truth_settings' in given_config:
-        new_config['truth_settings'] = deepcopy(given_config['truth_settings'])
+    if 'truth_settings' in new_config:
+        updated_config['truth_settings'] = deepcopy(new_config['truth_settings'])
 
-    if not isinstance(new_config['truth_settings'], list):
-        new_config['truth_settings'] = [new_config['truth_settings']]
+    if not isinstance(updated_config['truth_settings'], list):
+        updated_config['truth_settings'] = [updated_config['truth_settings']]
 
     default = deepcopy(config['truth_settings'])
     if not isinstance(default, list):
         default = [default]
 
-    new_config['truth_settings'] = update_truth_settings(new_config['truth_settings'], default)
+    updated_config['truth_settings'] = update_truth_settings(updated_config['truth_settings'], default)
 
     # Check for required keys
     for key in REQUIRED_CONFIGS:
-        if key not in new_config:
+        if key not in updated_config:
             raise SonusAIError(f'Missing required config in {name}: {key}')
 
-    return new_config
+    # Check for non-empty spectral masks
+    if len(updated_config['spectral_masks']) == 0:
+        updated_config['spectral_masks'] = config['spectral_masks']
+
+    return updated_config
 
 
 def update_truth_settings(given: Union[List[Dict], Dict], default: List[Dict] = None) -> List[Dict]:
@@ -686,3 +691,18 @@ def generate_mixture_filename(mixid: int, padding: int) -> str:
     :return: Zero-padded mixture file name
     """
     return f'{mixid:0{padding}}.h5'
+
+
+def get_spectral_masks(config: dict) -> SpectralMasks:
+    """Get the list of spectral masks from a config
+
+    :param config: Config dictionary
+    :return: List of spectral masks
+    """
+    from sonusai import SonusAIError
+    from sonusai.utils import dataclass_from_dict
+
+    try:
+        return dataclass_from_dict(SpectralMasks, config['spectral_masks'])
+    except Exception as e:
+        raise SonusAIError(f'Error in spectral_masks: {e}')
