@@ -13,8 +13,7 @@ def get_feature_from_audio(audio: AudioT, feature: str) -> Feature:
     mixdb = MixtureDatabase(config=MixtureDatabaseConfig(feature=feature,
                                                          num_classes=1,
                                                          truth_mutex=False,
-                                                         truth_reduction_function='max'),
-                            lazy_load=True)
+                                                         truth_reduction_function='max'))
 
     audio = np.pad(array=audio, pad_width=(0, get_pad_length(len(audio), mixdb.feature_step_samples)))
     mixdb.mixtures = [MRecord(samples=len(audio))]
