@@ -22,6 +22,7 @@ from sonusai.mixture.types import Location
 from sonusai.mixture.types import NoiseFiles
 from sonusai.mixture.types import Optional
 from sonusai.mixture.types import Segsnr
+from sonusai.mixture.types import SpectralMasks
 from sonusai.mixture.types import TargetFiles
 from sonusai.mixture.types import Truth
 from sonusai.mixture.types import TruthSettings
@@ -47,6 +48,8 @@ class MRecord(DataClassSonusAIMixin):
     random_snr: Optional[bool] = None
     samples: int = None
     snr: float = None
+    spectral_mask_index: Optional[int] = None
+    spectral_mask_seed: Optional[int] = None
     target_augmentation_index: List[int] = None
     target_file_index: List[int] = None
     target_gain: List[int] = None
@@ -73,6 +76,7 @@ class MixtureDatabaseConfig(DataClassSonusAIMixin):
     random_snrs: Optional[List[str]] = None
     seed: Optional[int] = 0
     snrs: List[float] = None
+    spectral_masks: Optional[SpectralMasks] = None
     target_augmentations: Augmentations = None
     targets: TargetFiles = None
     truth_mutex: bool = None
@@ -84,6 +88,8 @@ class MixtureDatabaseConfig(DataClassSonusAIMixin):
 class TransformConfig:
     N: int
     R: int
+    bin_start: int
+    bin_end: int
     ttype: str
 
 
@@ -93,7 +99,7 @@ class MixtureDatabase:
 
     def __init__(self,
                  config: Union[Location, MixtureDatabaseConfig],
-                 lazy_load: bool = False,
+                 lazy_load: bool = True,
                  show_progress: bool = False):
         if isinstance(config, MixtureDatabaseConfig):
             self.config = config
@@ -108,14 +114,20 @@ class MixtureDatabase:
 
         self.ft_config = TransformConfig(N=self.fg.ftransform_N,
                                          R=self.fg.ftransform_R,
+                                         bin_start=self.fg.bin_start,
+                                         bin_end=self.fg.bin_end,
                                          ttype=self.fg.ftransform_ttype)
 
         self.eft_config = TransformConfig(N=self.fg.eftransform_N,
                                           R=self.fg.eftransform_R,
+                                          bin_start=self.fg.bin_start,
+                                          bin_end=self.fg.bin_end,
                                           ttype=self.fg.eftransform_ttype)
 
         self.it_config = TransformConfig(N=self.fg.itransform_N,
                                          R=self.fg.itransform_R,
+                                         bin_start=self.fg.bin_start,
+                                         bin_end=self.fg.bin_end,
                                          ttype=self.fg.itransform_ttype)
 
         self.show_progress = show_progress
@@ -179,6 +191,8 @@ class MixtureDatabase:
         return calculate_transform_from_audio(audio=audio,
                                               transform=ForwardTransform(N=self.ft_config.N,
                                                                          R=self.ft_config.R,
+                                                                         bin_start=self.ft_config.bin_start,
+                                                                         bin_end=self.ft_config.bin_end,
                                                                          ttype=self.ft_config.ttype))
 
     def inverse_transform(self, transform: AudioF, trim: bool = True) -> AudioT:
@@ -197,31 +211,33 @@ class MixtureDatabase:
         return calculate_audio_from_transform(data=transform,
                                               transform=InverseTransform(N=self.it_config.N,
                                                                          R=self.it_config.R,
+                                                                         bin_start=self.it_config.bin_start,
+                                                                         bin_end=self.it_config.bin_end,
                                                                          ttype=self.it_config.ttype),
                                               trim=trim)
 
     @property
-    def class_balancing(self):
+    def class_balancing(self) -> bool:
         return self.config.class_balancing
 
     @class_balancing.setter
-    def class_balancing(self, value):
+    def class_balancing(self, value: bool) -> None:
         self.config.class_balancing = value
 
     @property
-    def class_balancing_augmentation(self):
+    def class_balancing_augmentation(self) -> Augmentation:
         return self.config.class_balancing_augmentation
 
     @class_balancing_augmentation.setter
-    def class_balancing_augmentation(self, value):
+    def class_balancing_augmentation(self, value: Augmentation) -> None:
         self.config.class_balancing_augmentation = value
 
     @property
-    def class_labels(self):
+    def class_labels(self) -> List[str]:
         return self.config.class_labels
 
     @class_labels.setter
-    def class_labels(self, value):
+    def class_labels(self, value: List[str]) -> None:
         from sonusai import SonusAIError
 
         if value is not None and (not isinstance(value, list) or len(value) != self.num_classes):
@@ -229,11 +245,11 @@ class MixtureDatabase:
         self.config.class_labels = value
 
     @property
-    def class_weights_threshold(self):
+    def class_weights_threshold(self) -> List[float]:
         return self.config.class_weights_threshold
 
     @class_weights_threshold.setter
-    def class_weights_threshold(self, value):
+    def class_weights_threshold(self, value: List[float]) -> None:
         from sonusai import SonusAIError
 
         if len(value) not in [1, self.num_classes]:
@@ -241,149 +257,157 @@ class MixtureDatabase:
         self.config.class_weights_threshold = value
 
     @property
-    def exhaustive_noise(self):
+    def exhaustive_noise(self) -> bool:
         return self.config.exhaustive_noise
 
     @exhaustive_noise.setter
-    def exhaustive_noise(self, value):
+    def exhaustive_noise(self, value: bool) -> None:
         self.config.exhaustive_noise = value
 
     @property
-    def feature(self):
+    def feature(self) -> str:
         return self.config.feature
 
     @feature.setter
-    def feature(self, value):
+    def feature(self, value: str) -> None:
         self.config.feature = value
 
     @property
-    def transform_frame_ms(self):
+    def transform_frame_ms(self) -> float:
         from sonusai.mixture import SAMPLE_RATE
 
         return float(self.ft_config.R) / float(SAMPLE_RATE / 1000)
 
     @property
-    def feature_ms(self):
+    def feature_ms(self) -> float:
         return self.transform_frame_ms * self.fg.decimation * self.fg.stride
 
     @property
-    def feature_samples(self):
+    def feature_samples(self) -> int:
         return self.ft_config.R * self.fg.decimation * self.fg.stride
 
     @property
-    def feature_step_ms(self):
+    def feature_step_ms(self) -> float:
         return self.transform_frame_ms * self.fg.decimation * self.fg.step
 
     @property
-    def feature_step_samples(self):
+    def feature_step_samples(self) -> int:
         return self.ft_config.R * self.fg.decimation * self.fg.step
 
     @property
-    def first_cba_index(self):
+    def first_cba_index(self) -> int:
         return self.config.first_cba_index
 
     @first_cba_index.setter
-    def first_cba_index(self, value):
+    def first_cba_index(self, value: int) -> None:
         self.config.first_cba_index = value
 
     @property
-    def ir_files(self):
+    def ir_files(self) -> ImpulseResponseFiles:
         return self.config.ir_files
 
     @property
-    def mixtures(self):
+    def mixtures(self) -> MRecords:
         return self.config.mixtures
 
     @mixtures.setter
-    def mixtures(self, value):
+    def mixtures(self, value: MRecords) -> None:
         self.config.mixtures = value
 
     @property
-    def noise_augmentations(self):
+    def noise_augmentations(self) -> Augmentations:
         return self.config.noise_augmentations
 
     @noise_augmentations.setter
-    def noise_augmentations(self, value):
+    def noise_augmentations(self, value: Augmentations) -> None:
         self.config.noise_augmentations = value
 
     @property
-    def noises(self):
+    def noises(self) -> NoiseFiles:
         return self.config.noises
 
     @noises.setter
-    def noises(self, value):
+    def noises(self, value: NoiseFiles) -> None:
         self.config.noises = value
 
     @property
-    def num_classes(self):
+    def num_classes(self) -> int:
         return self.config.num_classes
 
     @num_classes.setter
-    def num_classes(self, value):
+    def num_classes(self, value: int) -> None:
         self.config.num_classes = value
 
     @property
-    def random_snrs(self):
+    def random_snrs(self) -> List[str]:
         return self.config.random_snrs
 
     @random_snrs.setter
-    def random_snrs(self, value):
+    def random_snrs(self, value: List[str]) -> None:
         self.config.random_snrs = value
 
     @property
-    def seed(self):
+    def seed(self) -> int:
         return self.config.seed
 
     @seed.setter
-    def seed(self, value):
+    def seed(self, value: int) -> None:
         self.config.seed = value
 
     @property
-    def snrs(self):
+    def snrs(self) -> List[float]:
         return self.config.snrs
 
     @snrs.setter
-    def snrs(self, value):
+    def snrs(self, value: List[float]) -> None:
         self.config.snrs = value
 
     @property
-    def target_augmentations(self):
+    def spectral_masks(self) -> SpectralMasks:
+        return self.config.spectral_masks
+
+    @spectral_masks.setter
+    def spectral_masks(self, value: SpectralMasks) -> None:
+        self.config.spectral_masks = value
+
+    @property
+    def target_augmentations(self) -> Augmentations:
         return self.config.target_augmentations
 
     @target_augmentations.setter
-    def target_augmentations(self, value):
+    def target_augmentations(self, value: Augmentations) -> None:
         self.config.target_augmentations = value
 
     @property
-    def targets(self):
+    def targets(self) -> TargetFiles:
         return self.config.targets
 
     @targets.setter
-    def targets(self, value):
+    def targets(self, value: TargetFiles) -> None:
         self.config.targets = value
 
     @property
-    def truth_mutex(self):
+    def truth_mutex(self) -> bool:
         return self.config.truth_mutex
 
     @truth_mutex.setter
-    def truth_mutex(self, value):
+    def truth_mutex(self, value: bool) -> None:
         self.config.truth_mutex = value
 
     @property
-    def truth_reduction_function(self):
+    def truth_reduction_function(self) -> str:
         return self.config.truth_reduction_function
 
     @truth_reduction_function.setter
-    def truth_reduction_function(self, value):
+    def truth_reduction_function(self, value: str) -> None:
         self.config.truth_reduction_function = value
 
     @property
-    def truth_settings(self):
+    def truth_settings(self) -> TruthSettings:
         return self.config.truth_settings
 
     @truth_settings.setter
-    def truth_settings(self, value):
+    def truth_settings(self, value: TruthSettings) -> None:
         self.config.truth_settings = value
 
     @property
@@ -657,8 +681,8 @@ class MixtureDatabase:
         mrecord = self.mixtures[mixid]
         targets = []
         for idx in range(len(mrecord.target_file_index)):
-            if self.raw_target_audios is not None:
-                target = self.raw_target_audios[mrecord.target_file_index[idx]]
+            if self._raw_target_audios is not None:
+                target = self._raw_target_audios[mrecord.target_file_index[idx]]
             else:
                 target = read_audio(self.targets[mrecord.target_file_index[idx]].name)
 
@@ -759,8 +783,8 @@ class MixtureDatabase:
                 return noise
 
         mrecord = self.mixtures[mixid]
-        if self.augmented_noise_audios is not None:
-            noise = self.augmented_noise_audios[mrecord.noise_file_index][mrecord.noise_augmentation_index]
+        if self._augmented_noise_audios is not None:
+            noise = self._augmented_noise_audios[mrecord.noise_file_index][mrecord.noise_augmentation_index]
         else:
             noise = apply_augmentation(audio=read_audio(self.noises[mrecord.noise_file_index].name),
                                        augmentation=self.noise_augmentations[mrecord.noise_augmentation_index])
@@ -919,7 +943,11 @@ class MixtureDatabase:
         if noise is None:
             noise = self.mixture_noise(mixid=mixid)
 
-        fft = ForwardTransform(N=self.ft_config.N, R=self.ft_config.R, ttype=self.ft_config.ttype)
+        fft = ForwardTransform(N=self.ft_config.N,
+                               R=self.ft_config.R,
+                               bin_start=self.ft_config.bin_start,
+                               bin_end=self.ft_config.bin_end,
+                               ttype=self.ft_config.ttype)
 
         segsnr_t = np.empty(self.mixture_samples(mixid), dtype=np.float32)
 
@@ -995,6 +1023,7 @@ class MixtureDatabase:
         """
         import numpy as np
 
+        from sonusai.mixture import apply_spectral_mask
         from sonusai.mixture import truth_reduction
 
         if not force:
@@ -1038,57 +1067,62 @@ class MixtureDatabase:
             if np.isreal(truth_f).all():
                 truth_f = np.float32(np.real(truth_f))
 
+            if mrecord.spectral_mask_index is not None:
+                feature = apply_spectral_mask(feature=feature,
+                                              spectral_mask=self.spectral_masks[mrecord.spectral_mask_index],
+                                              seed=mrecord.spectral_mask_seed)
+
             return feature, truth_f
 
-        def mixture_feature(self,
-                            mixid: int,
-                            targets: AudiosT = None,
-                            noise: AudioT = None,
-                            mixture: AudioT = None,
-                            truth_t: Truth = None,
-                            force: bool = False) -> Feature:
-            """Get the feature data for the given mixid
+    def mixture_feature(self,
+                        mixid: int,
+                        targets: AudiosT = None,
+                        noise: AudioT = None,
+                        mixture: AudioT = None,
+                        truth_t: Truth = None,
+                        force: bool = False) -> Feature:
+        """Get the feature data for the given mixid
 
-            :param mixid: Mixture ID
-            :param targets: List of augmented target audio data (one per target in the mixup) for the given mixid
-            :param noise: Augmented noise audio data for the given mixid
-            :param mixture: Mixture audio data for the given mixid
-            :param truth_t: truth_t for the given mixid
-            :param force: Force computing data from original sources regardless of whether or not cached data exists
-            :return: Feature data
-            """
-            feature, _ = self.mixture_ft(mixid=mixid,
-                                         targets=targets,
-                                         noise=noise,
-                                         mixture=mixture,
-                                         truth_t=truth_t,
-                                         force=force)
-            return feature
+        :param mixid: Mixture ID
+        :param targets: List of augmented target audio data (one per target in the mixup) for the given mixid
+        :param noise: Augmented noise audio data for the given mixid
+        :param mixture: Mixture audio data for the given mixid
+        :param truth_t: truth_t for the given mixid
+        :param force: Force computing data from original sources regardless of whether or not cached data exists
+        :return: Feature data
+        """
+        feature, _ = self.mixture_ft(mixid=mixid,
+                                     targets=targets,
+                                     noise=noise,
+                                     mixture=mixture,
+                                     truth_t=truth_t,
+                                     force=force)
+        return feature
 
-        def mixture_truth_f(self,
-                            mixid: int,
-                            targets: AudiosT = None,
-                            noise: AudioT = None,
-                            mixture: AudioT = None,
-                            truth_t: Truth = None,
-                            force: bool = False) -> Truth:
-            """Get the truth_f data for the given mixid
+    def mixture_truth_f(self,
+                        mixid: int,
+                        targets: AudiosT = None,
+                        noise: AudioT = None,
+                        mixture: AudioT = None,
+                        truth_t: Truth = None,
+                        force: bool = False) -> Truth:
+        """Get the truth_f data for the given mixid
 
-            :param mixid: Mixture ID
-            :param targets: List of augmented target audio data (one per target in the mixup) for the given mixid
-            :param noise: Augmented noise audio data for the given mixid
-            :param mixture: Mixture audio data for the given mixid
-            :param truth_t: truth_t for the given mixid
-            :param force: Force computing data from original sources regardless of whether or not cached data exists
-            :return: truth_f data
-            """
-            _, truth_f = self.mixture_ft(mixid=mixid,
-                                         targets=targets,
-                                         noise=noise,
-                                         mixture=mixture,
-                                         truth_t=truth_t,
-                                         force=force)
-            return truth_f
+        :param mixid: Mixture ID
+        :param targets: List of augmented target audio data (one per target in the mixup) for the given mixid
+        :param noise: Augmented noise audio data for the given mixid
+        :param mixture: Mixture audio data for the given mixid
+        :param truth_t: truth_t for the given mixid
+        :param force: Force computing data from original sources regardless of whether or not cached data exists
+        :return: truth_f data
+        """
+        _, truth_f = self.mixture_ft(mixid=mixid,
+                                     targets=targets,
+                                     noise=noise,
+                                     mixture=mixture,
+                                     truth_t=truth_t,
+                                     force=force)
+        return truth_f
 
     def mixture_class_count(self,
                             mixid: int,
