@@ -316,8 +316,7 @@ def main():
         mixdb = MixtureDatabase(config=MixtureDatabaseConfig(feature=model.feature,
                                                              num_classes=model.output_shape[-1],
                                                              truth_mutex=False,
-                                                             truth_reduction_function='max'),
-                                lazy_load=True)
+                                                             truth_reduction_function='max'))
         logger.debug(f'Audio samples      {len(mixture)}')
         logger.debug(f'Feature shape      {feature.shape}')
 
