@@ -4,12 +4,12 @@ from typing import Dict
 from typing import List
 
 from sonusai.mixture.mixdb import MixtureDatabase
-from sonusai.mixture.mixdb import MixtureID
+from sonusai.mixture.types import GeneralizedIDs
 
 
 def get_mixids_from_mixture_field_predicate(mixdb: MixtureDatabase,
                                             field: str,
-                                            mixid: MixtureID = None,
+                                            mixids: GeneralizedIDs = None,
                                             predicate: Callable[[Any], bool] = None) -> Dict[int, List[int]]:
     """
     Generate mixture IDs based on mixture field and predicate
@@ -19,9 +19,7 @@ def get_mixids_from_mixture_field_predicate(mixdb: MixtureDatabase,
     """
     from typing import Any
 
-    from sonusai.mixture import convert_mixid_to_list
-
-    mixid_out = convert_mixid_to_list(mixdb, mixid)
+    mixid_out = mixdb.mixids_to_list(mixids)
 
     if predicate is None:
         def predicate(_: Any) -> bool:
@@ -39,9 +37,9 @@ def get_mixids_from_mixture_field_predicate(mixdb: MixtureDatabase,
                 criteria.add(value)
     criteria = sorted(list(criteria))
 
-    result = dict()
+    result = {}
     for criterion in criteria:
-        result[criterion] = list()
+        result[criterion] = []
         for i, x in enumerate(mixdb.mixtures):
             if i in mixid_out:
                 value = getattr(x, field)
@@ -57,7 +55,7 @@ def get_mixids_from_mixture_field_predicate(mixdb: MixtureDatabase,
 
 def get_mixids_from_truth_settings_field_predicate(mixdb: MixtureDatabase,
                                                    field: str,
-                                                   mixid: MixtureID = None,
+                                                   mixids: GeneralizedIDs = None,
                                                    predicate: Callable[[Any], bool] = None) -> Dict[int, List[int]]:
     """
     Generate mixture IDs based on target truth_settings field and predicate
@@ -67,9 +65,7 @@ def get_mixids_from_truth_settings_field_predicate(mixdb: MixtureDatabase,
     """
     from typing import Any
 
-    from sonusai.mixture import convert_mixid_to_list
-
-    mixid_out = convert_mixid_to_list(mixdb, mixid)
+    mixid_out = mixdb.mixids_to_list(mixids)
 
     # Get all field values
     values = get_all_truth_settings_values_from_field(mixdb, field)
@@ -81,17 +77,17 @@ def get_mixids_from_truth_settings_field_predicate(mixdb: MixtureDatabase,
     # Get only values of interest
     values = [value for value in values if predicate(value)]
 
-    result = dict()
+    result = {}
     for value in values:
         # Get a list of targets for each field value
-        indices = list()
+        indices = []
         for i, target in enumerate(mixdb.targets):
             for truth_setting in target.truth_settings:
                 if value in getattr(truth_setting, field):
                     indices.append(i)
         indices = sorted(list(set(indices)))
 
-        mixids = list()
+        mixids = []
         for index in indices:
             mixids.extend([i for i, x in enumerate(mixdb.mixtures) if
                            index in x.target_file_index and i in mixid_out])
@@ -107,7 +103,7 @@ def get_all_truth_settings_values_from_field(mixdb: MixtureDatabase, field: str)
     """
     Generate a list of all values corresponding to the given field in truth_settings
     """
-    result = list()
+    result = []
     for target in mixdb.targets:
         for truth_setting in target.truth_settings:
             value = getattr(truth_setting, field)
@@ -119,7 +115,7 @@ def get_all_truth_settings_values_from_field(mixdb: MixtureDatabase, field: str)
 
 
 def get_mixids_from_noise(mixdb: MixtureDatabase,
-                          mixid: MixtureID = None,
+                          mixids: GeneralizedIDs = None,
                           predicate: Callable[[Any], bool] = None) -> Dict[int, List[int]]:
     """
     Generate mixids based on noise index predicate
@@ -128,13 +124,13 @@ def get_mixids_from_noise(mixdb: MixtureDatabase,
         - values are lists of the mixids that match the noise index
     """
     return get_mixids_from_mixture_field_predicate(mixdb=mixdb,
-                                                   mixid=mixid,
+                                                   mixids=mixids,
                                                    field='noise_file_index',
                                                    predicate=predicate)
 
 
 def get_mixids_from_noise_augmentation(mixdb: MixtureDatabase,
-                                       mixid: MixtureID = None,
+                                       mixids: GeneralizedIDs = None,
                                        predicate: Callable[[Any], bool] = None) -> Dict[int, List[int]]:
     """
     Generate mixids based on a noise augmentation index predicate
@@ -143,13 +139,13 @@ def get_mixids_from_noise_augmentation(mixdb: MixtureDatabase,
         - values are lists of the mixids that match the noise augmentation index
     """
     return get_mixids_from_mixture_field_predicate(mixdb=mixdb,
-                                                   mixid=mixid,
+                                                   mixids=mixids,
                                                    field='noise_augmentation_index',
                                                    predicate=predicate)
 
 
 def get_mixids_from_target(mixdb: MixtureDatabase,
-                           mixid: MixtureID = None,
+                           mixids: GeneralizedIDs = None,
                            predicate: Callable[[Any], bool] = None) -> Dict[int, List[int]]:
     """
     Generate mixids based on a target index predicate
@@ -158,13 +154,13 @@ def get_mixids_from_target(mixdb: MixtureDatabase,
         - values are lists of the mixids that match the target index
     """
     return get_mixids_from_mixture_field_predicate(mixdb=mixdb,
-                                                   mixid=mixid,
+                                                   mixids=mixids,
                                                    field='target_file_index',
                                                    predicate=predicate)
 
 
 def get_mixids_from_target_augmentation(mixdb: MixtureDatabase,
-                                        mixid: MixtureID = None,
+                                        mixids: GeneralizedIDs = None,
                                         predicate: Callable[[Any], bool] = None) -> Dict[int, List[int]]:
     """
     Generate mixids based on a target augmentation index predicate
@@ -173,13 +169,13 @@ def get_mixids_from_target_augmentation(mixdb: MixtureDatabase,
         - values are lists of the mixids that match the target augmentation index
     """
     return get_mixids_from_mixture_field_predicate(mixdb=mixdb,
-                                                   mixid=mixid,
+                                                   mixids=mixids,
                                                    field='target_augmentation_index',
                                                    predicate=predicate)
 
 
 def get_mixids_from_snr(mixdb: MixtureDatabase,
-                        mixid: MixtureID = None,
+                        mixids: GeneralizedIDs = None,
                         predicate: Callable[[Any], bool] = None) -> Dict[float, List[int]]:
     """
     Generate mixids based on an SNR predicate
@@ -189,9 +185,7 @@ def get_mixids_from_snr(mixdb: MixtureDatabase,
     """
     from typing import Any
 
-    from sonusai.mixture import convert_mixid_to_list
-
-    mixid_out = convert_mixid_to_list(mixdb, mixid)
+    mixid_out = mixdb.mixids_to_list(mixids)
 
     # Get all the SNRs
     snrs = mixdb.snrs
@@ -203,7 +197,7 @@ def get_mixids_from_snr(mixdb: MixtureDatabase,
     # Get only the SNRs of interest (filter on predicate)
     snrs = [snr for snr in snrs if predicate(snr)]
 
-    result = dict()
+    result = {}
     for snr in snrs:
         # Get a list of mixids for each SNR
         result[snr] = sorted(
@@ -213,7 +207,7 @@ def get_mixids_from_snr(mixdb: MixtureDatabase,
 
 
 def get_mixids_from_truth_index(mixdb: MixtureDatabase,
-                                mixid: MixtureID = None,
+                                mixids: GeneralizedIDs = None,
                                 predicate: Callable[[Any], bool] = None) -> Dict[int, List[int]]:
     """
     Generate mixids based on a truth index predicate
@@ -222,13 +216,13 @@ def get_mixids_from_truth_index(mixdb: MixtureDatabase,
         - values are lists of the mixids that match the truth index
     """
     return get_mixids_from_truth_settings_field_predicate(mixdb=mixdb,
-                                                          mixid=mixid,
+                                                          mixids=mixids,
                                                           field='index',
                                                           predicate=predicate)
 
 
 def get_mixids_from_truth_function(mixdb: MixtureDatabase,
-                                   mixid: MixtureID = None,
+                                   mixids: GeneralizedIDs = None,
                                    predicate: Callable[[Any], bool] = None) -> Dict[int, List[int]]:
     """
     Generate mixids based on a truth function predicate
@@ -237,6 +231,6 @@ def get_mixids_from_truth_function(mixdb: MixtureDatabase,
         - values are lists of the mixids that match the truth function
     """
     return get_mixids_from_truth_settings_field_predicate(mixdb=mixdb,
-                                                          mixid=mixid,
+                                                          mixids=mixids,
                                                           field='function',
                                                           predicate=predicate)
