@@ -1,5 +1,7 @@
 # SonusAI general utilities
 from sonusai.utils.asl_p56 import asl_p56
+from sonusai.utils.asr import ASRResult
+from sonusai.utils.asr import calc_asr
 from sonusai.utils.braced_glob import braced_glob
 from sonusai.utils.calculate_input_shape import calculate_input_shape
 from sonusai.utils.create_ts_name import create_ts_name
@@ -13,6 +15,7 @@ from sonusai.utils.grouper import grouper
 from sonusai.utils.human_readable_size import human_readable_size
 from sonusai.utils.keras_utils import check_keras_overrides
 from sonusai.utils.keras_utils import create_onnx_from_keras
+from sonusai.utils.keras_utils import import_and_check_keras_model
 from sonusai.utils.keras_utils import import_keras_model
 from sonusai.utils.keras_utils import keras_onnx
 from sonusai.utils.numeric_conversion import float_to_int16
