@@ -11,28 +11,6 @@ from typing import Iterable
 from typing import List
 
 
-def initialize_map_func(ordered: bool, **kwargs: Any) -> Callable:
-    import multiprocessing as mp
-
-    # Extract num_cpus
-    num_cpus = kwargs.pop('num_cpus', None)
-
-    # Determine num_cpus
-    if num_cpus is None:
-        num_cpus = mp.cpu_count()
-    elif type(num_cpus) == float:
-        num_cpus = int(round(num_cpus * mp.cpu_count()))
-
-    # Create parallel generator
-    initializer = kwargs.pop('initializer', None)
-    initargs = kwargs.pop('initargs', None)
-    map_type = 'imap' if ordered else 'uimap'
-    pool = mp.Pool(num_cpus, initializer=initializer, initargs=initargs)
-    map_func = getattr(pool, map_type)
-
-    return map_func
-
-
 def _parallel(ordered: bool, function: Callable, *iterables: Iterable, **kwargs: Any) -> Generator:
     """Returns a generator for a parallel map.
 
@@ -45,10 +23,26 @@ def _parallel(ordered: bool, function: Callable, *iterables: Iterable, **kwargs:
         A generator which will apply the function to each element of the given Iterables
         in parallel in order.
     """
-    map_func = initialize_map_func(ordered, **kwargs)
+    import multiprocessing as mp
 
-    for item in map_func(function, *iterables):
-        yield item
+    # Extract num_cpus
+    num_cpus = kwargs.pop('num_cpus', None)
+
+    # Determine num_cpus
+    if num_cpus is None:
+        num_cpus = mp.cpu_count()
+    elif type(num_cpus) == float:
+        num_cpus = int(round(num_cpus * mp.cpu_count()))
+
+    # Create parallel generator
+    initializer = kwargs.pop('initializer', None)
+    initargs = kwargs.pop('initargs', None)
+    map_type = 'imap' if ordered else 'imap_unordered'
+    with mp.Pool(num_cpus, initializer=initializer, initargs=initargs) as pool:
+        map_func = getattr(pool, map_type)
+
+        for item in map_func(function, *iterables):
+            yield item
 
 
 def p_map(function: Callable, *iterables: Iterable, **kwargs: Any) -> List[Any]:
